function [J,G] = functional(mom,x,y,wx,wy,T,sigmaV2,sigmaI2,gammaR,gtabx,gtaby,sqnormy)

% functional for measure matching algorithm
%
% author : Joan Glaunès - joan.glaunes@free.fr

% list of arguments
if nargin == 0
    J = 'x,y,wx,wy,T,sigmaV2,sigmaI2,gammaR,gtabx,gtaby,sqnormy';
    return
end

tau = 1/(T-1);
nx = size(x,2);
ny = size(y,2);
y1 = y(1,:);
y2 = y(2,:);
y3 = y(3,:);

ng = length(gtabx)-1;

mom1 = mom(:,:,1);
mom2 = mom(:,:,2);
mom3 = mom(:,:,3);

[X1,X2,X3] = comptraj(x,mom,T,sigmaV2);

phiX1 = X1(:,T);
phiX2 = X2(:,T);
phiX3 = X3(:,T);

% compute energy term
E = gammaR * sum(sum(...
    mom1.*(X1(:,2:T)-X1(:,1:T-1)) + ...
    mom2.*(X2(:,2:T)-X2(:,1:T-1)) + ...
    mom3.*(X3(:,2:T)-X3(:,1:T-1))));

% compute matching term
R = matching(phiX1,phiX2,phiX3,wx,gtabx,y1,y2,y3,wy,gtaby,sigmaI2,sqnormy);

J = R + E;

%%%%%%%%%%%%%%%%% Gradient %%%%%%%%%%%%%%%%%%%%%%%%%%

if nargout > 1
    G = grad(mom,x,y,wx,wy,T,sigmaV2,sigmaI2,gammaR,gtabx,gtaby,sqnormy,X1,X2,X3);
end
