
function s = shooting(s)

% geodesic shooting for measure matching
%
% author : Joan Glaunès - joan.glaunes@free.fr

loadstruct('s')

mom1 = zeros(nx,T-1);
mom2 = zeros(nx,T-1);
mom3 = zeros(nx,T-1);

mom1(:,1) = mom(:,1,1);
mom2(:,1) = mom(:,1,2);
mom3(:,1) = mom(:,1,3);

X1(:,2:T) = 0;
X2(:,2:T) = 0;
X3(:,2:T) = 0;

% initialisation

distX1t = repmat(X1(:,1),1,nx)-repmat(X1(:,1)',nx,1);
distX2t = repmat(X2(:,1),1,nx)-repmat(X2(:,1)',nx,1);
distX3t = repmat(X3(:,1),1,nx)-repmat(X3(:,1)',nx,1);

for t=1:T-2
   kerdistx = 1./(1-(-(distX1t.^2+distX2t.^2+distX3t.^2)/sigmaV2(t)));
   
   X1(:,t+1) = X1(:,t) + tau * sum(kerdistx.*repmat(mom1(:,t)',nx,1),2);
   X2(:,t+1) = X2(:,t) + tau * sum(kerdistx.*repmat(mom2(:,t)',nx,1),2);
   X3(:,t+1) = X3(:,t) + tau * sum(kerdistx.*repmat(mom3(:,t)',nx,1),2);
   
   distX1t = repmat(X1(:,t+1)',nx,1)-repmat(X1(:,t+1),1,nx);
   distX2t = repmat(X2(:,t+1)',nx,1)-repmat(X2(:,t+1),1,nx);
   distX3t = repmat(X3(:,t+1)',nx,1)-repmat(X3(:,t+1),1,nx);
   dkerdistx = 1./(1-(-(distX1t.^2+distX2t.^2+distX3t.^2)/sigmaV2(t))).^2;
   
   dkerdistxscalmom = dkerdistx.*(...
      repmat(mom1(:,t)',nx,1).*repmat(mom1(:,t),1,nx) + ...
      repmat(mom2(:,t)',nx,1).*repmat(mom2(:,t),1,nx) + ...
      repmat(mom3(:,t)',nx,1).*repmat(mom3(:,t),1,nx));

   mom1(:,t+1) = mom1(:,t) + tau * 2 * sum(dkerdistxscalmom.*distX1t,1)'/sigmaV2(t);
   mom2(:,t+1) = mom2(:,t) + tau * 2 * sum(dkerdistxscalmom.*distX2t,1)'/sigmaV2(t);
   mom3(:,t+1) = mom3(:,t) + tau * 2 * sum(dkerdistxscalmom.*distX3t,1)'/sigmaV2(t);
      
end

% finalisation

kerdistx = 1./(1-(-(distX1t.^2+distX2t.^2+distX3t.^2)/sigmaV2(T)));
X1(:,T) = X1(:,T-1) + tau * sum(kerdistx.*repmat(mom1(:,T-1)',nx,1),2);
X2(:,T) = X2(:,T-1) + tau * sum(kerdistx.*repmat(mom2(:,T-1)',nx,1),2);
X3(:,T) = X3(:,T-1) + tau * sum(kerdistx.*repmat(mom3(:,T-1)',nx,1),2);
phiX1 = X1(:,T)';
phiX2 = X2(:,T)';
phiX3 = X3(:,T)';

iter = 0;

mom(:,:,1) = mom1;
mom(:,:,2) = mom2;
mom(:,:,3) = mom3;

savestruct('s')
